local addonName, TF = ...

local TradeFill = LibStub("AceAddon-3.0"):GetAddon(addonName)
local Loc = LibStub("AceLocale-3.0"):GetLocale(addonName)

function TradeFill:InitializeHistory()
    for timeStamp, _ in pairs(TradeFill.history.profile) do
        local playerName = TradeFill.history.profile[timeStamp].player
        local targetName = TradeFill.history.profile[timeStamp].target

        TF.options.args.logbook.args[timeStamp] = {
            type = "group",
            name = date("%d %B, |cFFFFFFFF%H:%M|r", timeStamp),
            desc = date("%d-%m-%Y\n\n", timeStamp) .. targetName,
            order = -tonumber(timeStamp),
            args = {
                day = {
                    type = "execute",
                    name = Loc["BUTTON_DELETE_DAY"],
                    desc = Loc["BUTTON_DELETE_DAY_DESC"],
                    func = function() TradeFill:DeleteHistoryDay(timeStamp) end,
                    confirm = true,
                    width = 0.5,
                },
            },
        }

        if string.find(targetName, "-") then
            local name = string.match(targetName, "^([^%-]+)")
            targetName = name .. "|r |cFFFFFFFF(*)|r"
        end

        TF.options.args.logbook.args[timeStamp].args.player = {
            type = "group",
            name = playerName,
            desc = "desc",
            order = 1,
            dialogInline = true,
            args = {
                player = {
                    type = "description",
                    name = " ",
                    order = 1,
                }
            },
        }

        TF.options.args.logbook.args[timeStamp].args.target = {
            type = "group",
            name = targetName,
            desc = "desc",
            order = 2,
            dialogInline = true,
            args = {
                target = {
                    type = "description",
                    name = " ",
                    order = 1,
                }
            },
        }

        TradeFill:GetHistory(timeStamp, "player",  "tradePlayer")
        TradeFill:GetHistory(timeStamp, "target",  "tradeTarget")
    end
end

function TradeFill:DeleteHistoryDay(timeStamp)
    self.history.profile[timeStamp] = nil
    TF.options.args.logbook.args[timeStamp] = nil

    if TF.data.setting.message then
        return
    end
    DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["REMOVED_TRADE"] .. date("%d %B, |cFFFFFFFF%H:%M|r", timeStamp))
end

function TradeFill:DeleteHistoryAll()
    for timeStamp, _ in pairs(TradeFill.history.profile) do
        TF.options.args.logbook.args[timeStamp] = nil
    end
    self.history:ResetProfile()

    if TF.data.setting.message then
        return
    end
    DEFAULT_CHAT_FRAME:AddMessage(TF.signature .. Loc["REMOVED_ALL"])
end

function TradeFill:GetHistory(timeStamp, unit, table)
    for tradedSlot, trade in pairs(TradeFill.history.profile[timeStamp][table]) do
        if tradedSlot == "money" and trade ~= "" then
            TF.options.args.logbook.args[timeStamp].args[unit].args["-1"]= {
                type = "description",
                fontSize = "medium",
                name = GetMoneyString(trade, 1000),
                order = 0,
            }
        end

        if type(tonumber(tradedSlot)) == "number" then
            TF.options.args.logbook.args[timeStamp].args[unit].args[tradedSlot]= {
                type = "description",
                fontSize = "small",
                name = "|T" .. GetItemIcon(trade.id) .. ":28[:28[:0:6|t " .. trade.quantity .. " x " .. TradeFill:GetItemColor(trade.quality, trade.name),
                order = tonumber(tradedSlot + 1),
            }
        end
    end
end

function TradeFill:setTradeHistory()
    if TF.tradeHistory and TF.tradeHistory.time and TF.tradeHistory.time ~= "" then
        local player = self.history.profile[TF.tradeHistory.time]

        player.player = TF.tradeHistory.player.name
        player.tradePlayer.money = TF.tradeHistory.player.money

        for i = 1, MAX_TRADABLE_ITEMS do
            if TF.tradeHistory.player[i] then
                for key, value in ipairs(TF.tradeHistory.player) do
                    local trade = player.tradePlayer[tostring(key)]

                    trade.id = value.id
                    trade.name = value.name
                    trade.texture = value.texture
                    trade.quantity = value.quantity
                    trade.quality = value.quality
                end
            end
        end

        player.target = TF.tradeHistory.target.name
        player.tradeTarget.money = TF.tradeHistory.target.money

        for i = 1, MAX_TRADABLE_ITEMS do
            if TF.tradeHistory.target[i] then
                for key, value in ipairs(TF.tradeHistory.target) do
                    local trade = player.tradeTarget[tostring(key)]

                    trade.id = value.id
                    trade.name = value.name
                    trade.texture = value.texture
                    trade.quantity = value.quantity
                    trade.quality = value.quality
                end
            end
        end
    end
    TradeFill:InitializeHistory()
end

function TradeFill:getTradeHistory()
    TF.tradeHistory = {
        time = "",
        player = {},
        target = {}
    }

    local player = TF.tradeHistory.player
    local target = TF.tradeHistory.target

    player.money = GetPlayerTradeMoney()
    target.money = GetTargetTradeMoney()

    for i = 1, MAX_TRADABLE_ITEMS do
        if GetTradePlayerItemLink(i) ~= nil then
            local name, texture, quantity, quality, _, _ = GetTradePlayerItemInfo(i)
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradePlayerItemLink(i))

            if name ~= nil then
                player[i] = {
                    id = id,
                    name = name,
                    texture = texture,
                    quantity = quantity,
                    quality = quality
                }
            end
        end

        if GetTradeTargetItemLink(i) ~= nil then
            local name, texture, quantity, quality, _, _ = GetTradeTargetItemInfo(i)
            local id, _, _, _, _, _, _ = GetItemInfoInstant(GetTradeTargetItemLink(i))

            if name ~= nil then
                target[i] = {
                    id = id,
                    name = name,
                    texture = texture,
                    quantity = quantity,
                    quality = quality
                }
            end
        end
    end

    if next(player) or next(target) or player.money > 0 or target.money > 0 then
        TF.tradeHistory.time = tostring(GetServerTime())
        player.name = TradeFill:setUnitName("player")
        target.name = TradeFill:setUnitName("npc")
    end
end

function TradeFill:setUnitName(unit)
    local _, classFile, _ = UnitClass(unit)
    local _, _, _, argbHex = GetClassColor(classFile)

    if classFile == "SHAMAN" then
        argbHex = "FF0070DE"
    end

    return "|c".. argbHex .. GetUnitName(unit, true) .. "|r"
end